//
// CTCSS Apllication
//
// The program is the application to generate CTCSS tones using a Arduino Nano.
// It uses a rotary encoder as input and a 8 x 2 LCD as output
//
// Copyright: 2016 PE1CID
// Version:
// 1.0 27-04-2016 Initial version based on T813 Tone
//

#include "Arduino.h"
#include <EEPROM.h>
#include <LiquidCrystal.h>
#include <CtcssDisplay.h>
#include <CtcssEeprom.h>
#include <CtcssEncoder.h>
#include <CtcssToneId.h>
#include <CtcssTone.h>

// Definitions
#define TONE_SAVE_TIMEOUT    (10000) // ms

// Static variables
static uint8_t       s_tone_val = 0;
static unsigned long s_current_time = 0;
static unsigned long s_previous_time = 0;
static unsigned long s_tone_save_timeout = 0;

void setup()
{
  uint8_t tone_val = 0;

  // initialize serial communication at 115200 bits per second:
  Serial.begin(115200);

  // Initialise the CTCSS tone generation
  CtcssTone.init();
  Serial.println("Tone init.");
  
  // inititalize the encoder
  CtcssEncoder.init();
  Serial.println("Encoder init.");
  
  // inititalize the display
  CtcssDisplay.init();
  Serial.println("Display init.");

  // Get the previously used tone value from the EEPROM
  tone_val = CtcssEeprom.get_start_tone();
  if ( tone_val >= TONE_MAX_TONES )
  {
    tone_val = 0;
  }
  // Start the tone
  if ( (Tone_Id)tone_val == tone_no_tone )
  {
    CtcssTone.tone_off();
  }
  else
  {
    CtcssTone.tone_on( (Tone_Id)tone_val );
  }

  // Display the tone
  CtcssDisplay.write_tone( (Tone_Id)tone_val );
}

void loop()
{
  E_Encoder_Values encoder_val;

  s_current_time = millis();
  if ( s_current_time != s_previous_time )
  {
    // Check if the encoder value has changed and handle it
    if ( CtcssEncoder.handle_encoder( &encoder_val ) )
    {
      // New encoder value
      switch( encoder_val )
      {
        case enc_no_action:
          Serial.println("Enc no action.");
          break;
        case enc_up:
          s_tone_val++;
          if ( s_tone_val >= TONE_MAX_TONES )
          {
            s_tone_val = 0;
          }
          Serial.print("Enc up. new val: ");
          Serial.print(s_tone_val);
          Serial.println(".");
          break;
        case enc_down:
          if ( s_tone_val > 0 )
          {
            s_tone_val--;
          }
          else
          {
            s_tone_val = TONE_MAX_TONES - 1;
          }
          Serial.print("Enc down. new val: ");
          Serial.print(s_tone_val);
          Serial.println(".");
          break;
        default:
          Serial.println("Unknown enc val.");
          break;
      }
      if ( (Tone_Id)s_tone_val == tone_no_tone )
      {
        CtcssTone.tone_off();
      }
      else
      {
        CtcssTone.tone_on( (Tone_Id)s_tone_val );
      }
      CtcssDisplay.write_tone( (Tone_Id)s_tone_val );
      s_tone_save_timeout = TONE_SAVE_TIMEOUT;
    }
    
    if ( s_tone_save_timeout > 0 )
    {
      s_tone_save_timeout--;
      if ( s_tone_save_timeout == 0 )
      {
        Serial.print("Save tone: ");
        Serial.print(s_tone_val);
        Serial.println(".");
        CtcssEeprom.set_start_tone( s_tone_val );
      }
    }
    
    s_previous_time = s_current_time;
  }
}
