/*
 * Driver to control the LCD module.
 *
 * Copyright: 2016 PE1CID
 * Version:
 * 1.0 16-04-2016 Initial version. Copied from the T813 project.
*/

/* Include files */
#include <LiquidCrystal.h>
#include "CtcssDisplay.h"
#include "CtcssToneId.h"

/* Definitions */

/* Display channel info defines */
#define TONE_TEXT_LINE       DISPLAY_LINE_1
#define TONE_TEXT_POS        0
#define TONE_ID_POS          6
#define TONE_FREQ_LINE       DISPLAY_LINE_2
#define TONE_FREQ_POS        0
#define MAX_TONE_ID          39
#define TONE_ID_LENGTH       3
#define TONE_FREQ_LENGTH     6

static const char tone_text [MAX_TONE_ID][TONE_ID_LENGTH] =
{
  "  ", " A", " B", " C", " D", " E", " F", " G", " H", " I",
  " J", " K", " L", " M", " N", " O", " P", " Q", " R", " S",
  " T", " U", " V", " W", " X", " Y", " Z", "AA", "AB", "AC",
  "AD", "AE", "AF", "AG", "AH", "AI", "AJ", "AK", "AL"
};

static const char tone_freq_text [MAX_TONE_ID][TONE_FREQ_LENGTH] =
{
  " OFF ", " 67,0", " 71,9", " 74,4", " 77,0", " 79,7", " 82,5", " 85,4", " 88,5", " 91,5",
  " 94,8", " 97,4", "100,0", "103,5", "170,2", "110,9", "114,8", "118,8", "123,0", "127,3",
  "131,8", "136,5", "141,3", "146,2", "151,4", "156,7", "162,2", "167,9", "173,8", "179,9",
  "186,2", "192,8", "203,5", "210,7", "218,1", "225,7", "233,6", "241,8", "250,3"
};

// Constructors ////////////////////////////////////////////////////////////////

CtcssDisplayClass::CtcssDisplayClass()
{
}

// initialize the library with the numbers of the interface pins
//                rs  en  d4  d5  d6  d7
LiquidCrystal lcd( 8,  9,  4,  5,  6,  7);

// Private Methods //////////////////////////////////////////////////////////////

// Public Methods //////////////////////////////////////////////////////////////

/*
 * init
 * Function to initialise the LCD module
 * in: none
 * out: none
 * return: none
 */
void CtcssDisplayClass::init ( void )
{
  // Set up the LCD's number of columns and rows:
  lcd.begin(8, 2);

  // Set the attributes of the display
  void noBlink();
  void noCursor();
  void clear();

  // Write the fixed text
  lcd.setCursor(0, DISPLAY_LINE_1);
  lcd.print("Toon: ");
  lcd.setCursor(5, DISPLAY_LINE_2);
  lcd.print(" Hz");
}

/*
 * write_tone
 * Function to write the tone to the LCD module.
 * in: tone_id
 * out: none
 * return: none
 */
void CtcssDisplayClass::write_tone ( Tone_Id tone_id )
{
  // Write the tone id
  lcd.setCursor( TONE_ID_POS, TONE_TEXT_LINE );
  lcd.print( tone_text[ tone_id ] );

  // Write the tone frequency
  lcd.setCursor( TONE_FREQ_POS, TONE_FREQ_LINE );
  lcd.print( tone_freq_text[ tone_id ] );
}

// Preinstantiate Objects //////////////////////////////////////////////////////

CtcssDisplayClass CtcssDisplay = CtcssDisplayClass();

/* End of file CtcssDisplay.cpp */
