/*
 * Driver to check and handle anencoder connected to D2 - D3.
 *
 * REMARK: No software debouncing used.
 *
 * Copyright: 2016 PE1CID
 * Version:
 * 1.0 27-04-2016 Initial version
 */

/* Include files */
#include "arduino.h"
#include <inttypes.h>
#include "CtcssEncoder.h"

/* Defintions */

typedef enum
{
    Encoder_No_Action,
    Encoder_CW_Action,
    Encoder_CCW_Action,
    Encoder_Unk_Action
} Encoder_Action;

const Encoder_Action encoder_action_table [4][4] =
{
    /* Prev val/new val 00                 01                  10                  11 */
    /* 00 */ { Encoder_No_Action,  Encoder_CW_Action,  Encoder_CCW_Action, Encoder_Unk_Action},
    /* 01 */ { Encoder_CCW_Action, Encoder_No_Action,  Encoder_Unk_Action, Encoder_CW_Action },
    /* 10 */ { Encoder_CW_Action,  Encoder_Unk_Action, Encoder_No_Action,  Encoder_CCW_Action },
    /* 11 */ { Encoder_Unk_Action, Encoder_CCW_Action, Encoder_CW_Action,  Encoder_No_Action }
};

/* set pin numbers */
const int InpEncA = 2;     // the number of the encoder pins
const int InpEncB = 3;

// Constructors ////////////////////////////////////////////////////////////////

CtcssEncoderClass::CtcssEncoderClass()
{
}

/* Initialise the class variables */

uint8_t CtcssEncoderClass::previous_encoder_value = 0;
int8_t  CtcssEncoderClass::rotary_divider         = 0;

// Public Methods //////////////////////////////////////////////////////////////

/*
 * init
 * Function to initialise the switches
 * in: none
 * out: none
 * return: none
 */
void CtcssEncoderClass::init ( void )
{
  // initialize the digital pins as an input with internal pullup.
  pinMode( InpEncA, INPUT_PULLUP );
  pinMode( InpEncB, INPUT_PULLUP );

  previous_encoder_value = ( digitalRead( InpEncB ) << 1 ) + digitalRead( InpEncA );
}

/*
 * handle_switches
 * Function to check if a switch has changed
 * in: none
 * out: p_switch_value, new switch value
 * return: true, when a new vlaue is read
 */
bool CtcssEncoderClass::handle_encoder ( E_Encoder_Values *p_encoder_value )
{
  bool           new_value = false;
  uint8_t        current_encoder_value = 0;
  Encoder_Action encoder_action        = Encoder_No_Action;

  /* Check and handle the encoder */
  *p_encoder_value = enc_no_action;
  current_encoder_value = ( digitalRead( InpEncB ) << 1 ) + digitalRead( InpEncA );
  encoder_action = encoder_action_table[previous_encoder_value][current_encoder_value];
  switch( encoder_action )
  {
    case Encoder_No_Action:
      /* No action required */
      break;
    case Encoder_CW_Action:
      rotary_divider++;
      if ( rotary_divider > 1 )
      {
        rotary_divider  = 0;
        *p_encoder_value = enc_up;
        new_value = true;
      }
      break;
    case Encoder_CCW_Action:
      rotary_divider--;
      if ( rotary_divider < -1 )
      {
        rotary_divider = 0;
        *p_encoder_value = enc_down;
        new_value = true;
      }
      break;
    case Encoder_Unk_Action:
      /* Both encoder channels changed, no action */
      break;
    default:
      /* ERROR: Invalid value of encoder_action, no action */
      current_encoder_value = 0;
      break;
  }
  previous_encoder_value = current_encoder_value;

  return( new_value );
}

// Preinstantiate Objects //////////////////////////////////////////////////////

CtcssEncoderClass CtcssEncoder = CtcssEncoderClass();

/* End of file CtcssEncoder.cpp */
