/*
 * Driver to check and handle the switches connected to D2 - D9.
 *
 * REMARK: No software debouncing used.
 *
 * Copyright: 2016 PE1CID
 * Version:
 * 1.0 06-01-2016 Initial version
 */

/* Include files */
#include "arduino.h"
#include <inttypes.h>
#include <Bounce2.h>
#include "CtcssSwitches.h"

/* Defintions */

#define DEBOUNCE_TIME    15

/* set pin numbers */
const int InpSw0 = 2;     // the number of the button pin
const int InpSw1 = 3;
const int InpSw2 = 4;
const int InpSw3 = 5;
const int InpSw4 = 6;
const int InpSw5 = 7;
const int InpSw6 = 8;
const int InpSw7 = 9;

// Constructors ////////////////////////////////////////////////////////////////

CtcssSwitchesClass::CtcssSwitchesClass()
{
}

// Instantiate a Bounce objects
Bounce BounceSw0    = Bounce();
Bounce BounceSw1    = Bounce();
Bounce BounceSw2    = Bounce();
Bounce BounceSw3    = Bounce();
Bounce BounceSw4    = Bounce();
Bounce BounceSw5    = Bounce();
Bounce BounceSw6    = Bounce();
Bounce BounceSw7    = Bounce();

/* Initialise the class variables */

uint8_t CtcssSwitchesClass::previous_switch_value = 0;

// Public Methods //////////////////////////////////////////////////////////////

/*
 * init
 * Function to initialise the switches
 * in: none
 * out: none
 * return: none
 */
void CtcssSwitchesClass::init ( void )
{
  // initialize the digital pins as an input with internal pullup.
  pinMode( InpSw0   , INPUT_PULLUP );
  pinMode( InpSw1   , INPUT_PULLUP );
  pinMode( InpSw2   , INPUT_PULLUP );
  pinMode( InpSw3   , INPUT_PULLUP );
  pinMode( InpSw4   , INPUT_PULLUP );
  pinMode( InpSw5   , INPUT_PULLUP );
  pinMode( InpSw6   , INPUT_PULLUP );
  pinMode( InpSw7   , INPUT_PULLUP );

  // After setting up the button, setup debouncer
  BounceSw0.attach( InpSw0 );
  BounceSw1.attach( InpSw1 );
  BounceSw2.attach( InpSw2 );
  BounceSw3.attach( InpSw3 );
  BounceSw4.attach( InpSw4 );
  BounceSw5.attach( InpSw5 );
  BounceSw6.attach( InpSw6 );
  BounceSw7.attach( InpSw7 );

  BounceSw0.interval( DEBOUNCE_TIME );
  BounceSw1.interval( DEBOUNCE_TIME );
  BounceSw2.interval( DEBOUNCE_TIME );
  BounceSw3.interval( DEBOUNCE_TIME );
  BounceSw4.interval( DEBOUNCE_TIME );
  BounceSw5.interval( DEBOUNCE_TIME );
  BounceSw6.interval( DEBOUNCE_TIME );
  BounceSw7.interval( DEBOUNCE_TIME );
}


/*
 * handle_switches
 * Function to check if a switch has changed
 * in: none
 * out: p_switch_value, new switch value
 * return: true, when a new vlaue is read
 */
bool CtcssSwitchesClass::handle_switches ( uint8_t *p_switch_value )
{
  bool new_value = false;
  uint8_t current_switch_value = 0;

  /* Check and handle the volume encoder */
  current_switch_value = (digitalRead( InpSw7 ) << 7) +
	                     (digitalRead( InpSw6 ) << 6) +
	                     (digitalRead( InpSw6 ) << 5) +
	                     (digitalRead( InpSw6 ) << 4) +
	                     (digitalRead( InpSw6 ) << 3) +
	                     (digitalRead( InpSw6 ) << 2) +
	                     (digitalRead( InpSw6 ) << 1) +
						 digitalRead( InpSw0 );
  if ( current_switch_value != previous_switch_value )
  {
    /* handle changed inputs */
	*p_switch_value = current_switch_value;
    previous_switch_value = current_switch_value;
	new_value = true;
  }

  return( new_value );
}

// Preinstantiate Objects //////////////////////////////////////////////////////

CtcssSwitchesClass CtcssSwitches = CtcssSwitchesClass();

/* End of file CtcssSwitches.cpp */
